/*****************************************************************************
 * Copyright (c) 2015 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *
 *		CEA LIST - Initial API and implementation
 *
 *****************************************************************************/
package org.eclipse.papyrus.sysml14.service.types.advice;

import java.util.Arrays;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.common.core.command.CompositeCommand;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.common.core.command.UnexecutableCommand;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.ISpecializationType;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.papyrus.sysml14.service.types.command.SetNestedPathCommand;
import org.eclipse.papyrus.sysml14.service.types.util.ConnectorUtil;
import org.eclipse.papyrus.sysml14.service.types.util.SysMLServiceTypeUtil;
import org.eclipse.papyrus.sysml14.service.types.util.UMLConnectorUtils;
import org.eclipse.papyrus.uml.service.types.element.UMLElementTypes;
import org.eclipse.papyrus.uml.service.types.utils.RequestParameterUtils;
import org.eclipse.uml2.uml.Property;

/**
 * 8.3.2.3 Block
 * the service edit must implement the following constraints
 * isEncapsulated: Boolean [0..1]
 * If true, then the block is treated as a black box; a part typed by this black box can only be connected via its ports or
 * directly to its outer boundary. If false, or if a value is not present, then connections can be established to elements of
 * its internal structure via deep-nested connector ends.
 * this class satisfies the requirement{@papyrus.req org.eclipse.papyrus.sysml14.service.types#Req_007}
 *
 */
public class ConnectorCreationEditHelperAdvice extends AbstractEditHelperAdvice {

	private UMLConnectorUtils utils = new UMLConnectorUtils();


	@Override
	protected ICommand getBeforeCreateRelationshipCommand(CreateRelationshipRequest request) {

		EObject source = request.getSource();
		EObject target = request.getTarget();
		View sourceView = RequestParameterUtils.getSourceView(request);
		View targetView = RequestParameterUtils.getTargetView(request);

		// Restrict this advice action to the end of Connector creation gesture (before clicking on target)
		// in order to add SysML specific constraint
		if ((Arrays.asList(request.getElementType().getAllSuperTypes()).contains(UMLElementTypes.CONNECTOR))
				&& (request.getSource() != null) && (request.getTarget() != null) && (request.getContainer() != null)) {

			// Restrict action to SysML Connector (meaning owned by Block)
			if (((ISpecializationType) ElementTypeRegistry.getInstance().getType(SysMLServiceTypeUtil.ORG_ECLIPSE_PAPYRUS_SYSML14_BLOCK)).getMatcher().matches(request.getContainer())) {

				// If the source or target view is enclosed in a structure encapsulated view, forbid creation.
				if (ConnectorUtil.isCrossingEncapsulation(sourceView, targetView)
						|| ConnectorUtil.isCrossingEncapsulation(targetView, sourceView)) {
					return UnexecutableCommand.INSTANCE;
				}
				if (UMLConnectorUtils.canCreate(source, target, sourceView, targetView)) {
					request.setParameter(org.eclipse.papyrus.uml.service.types.utils.RequestParameterConstants.UML_STRICT, false);
				}
			}
		}
		return super.getBeforeCreateRelationshipCommand(request);
	}

	@Override
	protected ICommand getAfterCreateRelationshipCommand(CreateRelationshipRequest request) {

		ICommand defaultCommand = super.getAfterCreateRelationshipCommand(request);

		View sourceView = RequestParameterUtils.getSourceView(request);
		View targetView = RequestParameterUtils.getTargetView(request);

		// Restrict this advice action to the end of Connector creation gesture (before clicking on target)
		// in order to add SysML specific constraint
		if ((Arrays.asList(request.getElementType().getAllSuperTypes()).contains(UMLElementTypes.CONNECTOR))
				&& (request.getSource() != null) && (request.getTarget() != null) && (request.getContainer() != null)) {

			// Restrict action to SysML Connector (meaning owned by Block)
			if (((ISpecializationType) ElementTypeRegistry.getInstance().getType(SysMLServiceTypeUtil.ORG_ECLIPSE_PAPYRUS_SYSML14_BLOCK)).getMatcher().matches(request.getContainer())) {

				// Check if source view is nested
				if (utils.isNestedConnectableElement(sourceView, targetView)) {
					List<Property> nestedPath = utils.getNestedPropertyPath(sourceView, targetView);
					SetNestedPathCommand next = new SetNestedPathCommand("Set connector nested source path", request.getNewElement(), request, nestedPath, SetNestedPathCommand.NESTED_SOURCE);
					defaultCommand = CompositeCommand.compose(defaultCommand, next);
				}

				// Check if target view is nested
				if (utils.isNestedConnectableElement(targetView, sourceView)) {
					List<Property> nestedPath = utils.getNestedPropertyPath(targetView, sourceView);
					SetNestedPathCommand next = new SetNestedPathCommand("Set connector nested source path", request.getNewElement(), request, nestedPath, SetNestedPathCommand.NESTED_TARGET);
					defaultCommand = CompositeCommand.compose(defaultCommand, next);
				}
			}
		}

		return defaultCommand;
	}
}
